<!DOCTYPE html>
<html>
  <head>
    <meta charset="UTF-8">
    <title>演示</title>
    <link rel="stylesheet" href="{{ static_url("聊天.css") }}" type="text/css">
    <link rel="stylesheet" href="https://code.jquery.com/ui/1.12.1/themes/base/jquery-ui.css">
    <script src="https://code.jquery.com/jquery-3.5.1.min.js" type="text/javascript"></script>
    <script src="https://code.jquery.com/ui/1.12.1/jquery-ui.min.js" integrity="sha256-VazP97ZCwtekAsvgPBSUwPFKdrwD3unUfSGVYrahUqU=" crossorigin="anonymous"></script>

    <link rel="stylesheet" href="{{ static_url("codemirror/核心/codemirror.css") }}">
    <script src="{{ static_url("codemirror/核心/codemirror.js") }}"></script>
    <script src="{{ static_url("codemirror/mode/clike/clike.js") }}"></script>
  </head>
  <body>
    <div id="详情">
      <textarea id="源码编辑"></textarea>
    </div>
    <div id="对话" style="float: left">
      <div id="历史">
        {% for 话语 in 所有对话 %}
          {% include "消息.html" %}
        {% end %}
      </div>
      <div id="请求">
          <input type="text" name="要求内容" id="输入" style="width:400px">
      </div>
    </div>
    <script>
      var 编辑器 = CodeMirror.fromTextArea(document.getElementById("源码编辑"), {
        mode : "text/x-java",
        lineNumbers : true,
        matchBrackets : true
      });
      编辑器.setSize(500, window.innerHeight - 20);
    </script>
    <script src="{{ static_url("聊天.js") }}" type="text/javascript"></script>
  </body>
</html>
