def 插入排序(数列):
  for i in range(1, len(数列)):
    挑出 = 数列[i]
    位置 = i - 1
    while (位置 >= 0 and 数列[位置] > 挑出):
      数列[位置 + 1] = 数列[位置]
      位置 -= 1
    数列[位置 + 1] = 挑出
乱序数列 = [5, 2, 1, 8, 4]
插入排序(乱序数列)
print(乱序数列)