#详情 {
  border:1px solid black;
  float: left;
}

#对话 {
  bottom: 20px;
  left: 20px;
  height: 770px;
  margin: 20px;
  overflow: auto;
  white-space: pre-wrap;
  width: calc(100% - 580px);
}

/* 置于顶部理由: 1) 详情内容多在顶部, 左右扫视方便. 2) 便于添加补全 */
#请求 {
  position: absolute;
  top: 10px;
}

#输入 {
  font-family: sans-serif;
  font-size: 15pt;
}

#历史 .回应 {
  border-radius: 10px;
  background: lightgreen;
  padding: 5px;
}
